@@ -2,7 +2,7 @@
  * (C) Copyright 2016
  * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
  *
- * SPDX-License-Identifier:     GPL-2.0-or-later
+ * SPDX-License-Identifier:     GPL-2.0-only
  */
 
 #include <stdio.h>
@@ -129,27 +129,18 @@ static int versions_settings(void *setting, void *data)
 	return 0;
 }
 
-void get_sw_versions(char *cfgname, struct swupdate_cfg *sw)
+void get_sw_versions(swupdate_cfg_handle *handle, struct swupdate_cfg *sw)
 {
-	int ret = -EINVAL;
-
-	/*
-	 * Try to read versions from configuration file
-	 * If not found, fall back to a legacy file
-	 * in the format "<image name> <version>"
-	 */
-	if (cfgname)
-		ret = read_module_settings(cfgname, "versions",
-						versions_settings,
-						sw);
-
-	if (ret)
-		ret = read_sw_version_file(sw);
-
+	/* Try to read versions from configuration file */
+	if (handle != NULL && read_module_settings(handle, "versions", versions_settings, sw) == 0) {
+		return;
+	}
+	/* If not found, fall back to a legacy file in the format "<image name> <version>" */
+	read_sw_version_file(sw);
 }
 #else
 
-void get_sw_versions(char __attribute__ ((__unused__)) *cfgname,
+void get_sw_versions(swupdate_cfg_handle  __attribute__ ((__unused__))*handle,
 			struct swupdate_cfg *sw)
 {
 	read_sw_version_file(sw);
