@@ -3,7 +3,7 @@
  * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
  * 	on behalf of ifm electronic GmbH
  *
- * SPDX-License-Identifier:     GPL-2.0-or-later
+ * SPDX-License-Identifier:     GPL-2.0-only
  */
 
 #include <stdio.h>
@@ -44,6 +44,7 @@
 #include "progress.h"
 #include "pctl.h"
 #include "state.h"
+#include "bootloader.h"
 
 #define BUFF_SIZE	 4096
 #define PERCENT_LB_INDEX	4
@@ -103,7 +104,7 @@ static int extract_file_to_tmp(int fd, const char *fname, unsigned long *poffs,
 		return -1;
 
 	if (copyfile(fd, &fdout, fdh.size, poffs, 0, 0, 0, &checksum, NULL,
-		     encrypted ? 1 : 0, NULL, NULL) < 0) {
+		     encrypted, NULL, NULL) < 0) {
 		close(fdout);
 		return -1;
 	}
@@ -121,7 +122,7 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 	int status = STREAM_WAIT_DESCRIPTION;
 	unsigned long offset;
 	struct filehdr fdh;
-	int skip;
+	swupdate_file_t skip;
 	uint32_t checksum;
 	int fdout;
 	struct img_type *img, *part;
@@ -221,7 +222,9 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 				fdout = openfileoutput(img->extract_file);
 				if (fdout < 0)
 					return -1;
-				if (copyfile(fd, &fdout, fdh.size, &offset, 0, 0, 0, &checksum, img->sha256, 0, NULL, NULL) < 0) {
+				if (!img_check_free_space(img, fdout))
+					return -1;
+				if (copyfile(fd, &fdout, fdh.size, &offset, 0, 0, 0, &checksum, img->sha256, false, NULL, NULL) < 0) {
 					close(fdout);
 					return -1;
 				}
@@ -233,7 +236,7 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 				break;
 
 			case SKIP_FILE:
-				if (copyfile(fd, &fdout, fdh.size, &offset, 0, skip, 0, &checksum, NULL, 0, NULL, NULL) < 0) {
+				if (copyfile(fd, &fdout, fdh.size, &offset, 0, skip, 0, &checksum, NULL, false, NULL, NULL) < 0) {
 					return -1;
 				}
 				if (!swupdate_verify_chksum(checksum, fdh.chksum)) {
@@ -249,8 +252,8 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 				 * just once
 				 */
 				if (!installed_directly) {
-					if (!software->globals.dry_run && software->bootloader_transaction_marker) {
-						save_state_string((char*)BOOTVAR_TRANSACTION, STATE_IN_PROGRESS);
+					if (!software->parms.dry_run && software->bootloader_transaction_marker) {
+						bootloader_env_set(BOOTVAR_TRANSACTION, get_state_string(STATE_IN_PROGRESS));
 					}
 					installed_directly = true;
 				}
@@ -263,7 +266,7 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 					if (!part->install_directly && part->is_partitioner) {
 						TRACE("Need to adjust partition %s before streaming %s",
 							part->volname, img->fname);
-						if (install_single_image(part, software->globals.dry_run)) {
+						if (install_single_image(part, software->parms.dry_run)) {
 							ERROR("Error adjusting partition %s", part->volname);
 							return -1;
 						}
@@ -272,7 +275,7 @@ static int extract_files(int fd, struct swupdate_cfg *software)
 					}
 				}
 				img->fdin = fd;
-				if (install_single_image(img, software->globals.dry_run)) {
+				if (install_single_image(img, software->parms.dry_run)) {
 					ERROR("Error streaming %s", img->fname);
 					return -1;
 				}
@@ -455,8 +458,16 @@ static int save_stream(int fdin, struct swupdate_cfg *software)
 	lseek(tmpfd, 0, SEEK_SET);
 
 	fdout = openfileoutput(software->output);
-	if (fdout < 0)
-		return -1;
+	/*
+	 * Try to create directory if file cannot be opened
+	 */
+	if (fdout < 0) {
+		if (mkpath(software->output, 0755))
+			return -1;
+		fdout = openfileoutput(software->output);
+		if (fdout < 0)
+			return -1;
+	}
 
 	ret = cpfiles(tmpfd, fdout, 0);
 	if (ret < 0)
@@ -487,6 +498,7 @@ void *network_initializer(void *data)
 	int ret;
 	struct swupdate_cfg *software = data;
 	struct swupdate_request *req;
+	struct swupdate_parms parms;
 
 	/* No installation in progress */
 	memset(&inst, 0, sizeof(inst));
@@ -497,10 +509,11 @@ void *network_initializer(void *data)
 	/* fork off the local dialogs and network service */
 	network_thread_id = start_thread(network_thread, &inst);
 
+	TRACE("Main loop daemon");
+	thread_ready();
 	/* handle installation requests (from either source) */
 	while (1) {
 		ret = 0;
-		TRACE("Main loop Daemon");
 
 		/* wait for someone to issue an install request */
 		pthread_mutex_lock(&stream_mutex);
@@ -508,22 +521,27 @@ void *network_initializer(void *data)
 		inst.status = RUN;
 		pthread_mutex_unlock(&stream_mutex);
 		notify(START, RECOVERY_NO_ERROR, INFOLEVEL, "Software Update started !");
+		TRACE("Software update started");
 
 		req = &inst.req;
 
+		/*
+		 * Save default values, they can be changed by a
+		 * install request
+		 */
+		parms = software->parms;
 		/*
 		 * Check if the dry run flag is overwritten
 		 */
 		switch (req->dry_run){
 		case RUN_DRYRUN:
-			software->globals.dry_run = 1;
+			software->parms.dry_run = true;
 			break;
 		case RUN_INSTALL:
-			software->globals.dry_run = 0;
+			software->parms.dry_run = false;
 			break;
 		case RUN_DEFAULT:
-		default:
-			software->globals.dry_run = software->globals.default_dry_run;
+			break;
 		}
 
 		/*
@@ -531,19 +549,14 @@ void *network_initializer(void *data)
 		 */
 		if ((strnlen(req->software_set, sizeof(req->software_set)) > 0) &&
 				(strnlen(req->running_mode, sizeof(req->running_mode)) > 0)) {
-			strlcpy(software->software_set, req->software_set, sizeof(software->software_set) - 1);
-			strlcpy(software->running_mode, req->running_mode, sizeof(software->running_mode) - 1);
-		} else {
-			strlcpy(software->software_set, software->globals.default_software_set,
-				sizeof(software->software_set) - 1);
-			strlcpy(software->running_mode, software->globals.default_running_mode,
-				sizeof(software->running_mode) - 1);
+			strlcpy(software->parms.software_set, req->software_set, sizeof(software->parms.software_set) - 1);
+			strlcpy(software->parms.running_mode, req->running_mode, sizeof(software->parms.running_mode) - 1);
 		}
 
 		/*
 		 * Check if the stream should be saved
 		 */
-		if (strlen(software->output)) {
+		if (!req->disable_store_swu  && strlen(software->output)) {
 			ret = save_stream(inst.fd, software);
 			if (ret < 0) {
 				notify(FAILURE, RECOVERY_ERROR, ERRORLEVEL,
@@ -582,19 +595,21 @@ void *network_initializer(void *data)
 			 * must be successful. Set we have
 			 * initiated an update
 			 */
-			if (!software->globals.dry_run && software->bootloader_transaction_marker) {
-				save_state_string((char*)BOOTVAR_TRANSACTION, STATE_IN_PROGRESS);
+			if (!software->parms.dry_run && software->bootloader_transaction_marker) {
+				bootloader_env_set(BOOTVAR_TRANSACTION, get_state_string(STATE_IN_PROGRESS));
 			}
 
 			notify(RUN, RECOVERY_NO_ERROR, INFOLEVEL, "Installation in progress");
-			ret = install_images(software, 0, 0);
+			ret = install_images(software);
 			if (ret != 0) {
-				if (!software->globals.dry_run && software->bootloader_transaction_marker) {
-					save_state_string((char*)BOOTVAR_TRANSACTION, STATE_FAILED);
+				if (!software->parms.dry_run && software->bootloader_transaction_marker) {
+					bootloader_env_set(BOOTVAR_TRANSACTION, get_state_string(STATE_FAILED));
 				}
 				notify(FAILURE, RECOVERY_ERROR, ERRORLEVEL, "Installation failed !");
 				inst.last_install = FAILURE;
-				if (!software->globals.dry_run && save_state((char *)STATE_KEY, STATE_FAILED) != SERVER_OK) {
+				if (!software->parms.dry_run
+				    && software->bootloader_state_marker
+				    && save_state(STATE_FAILED) != SERVER_OK) {
 					WARN("Cannot persistently store FAILED update state.");
 				}
 			} else {
@@ -602,10 +617,12 @@ void *network_initializer(void *data)
 				 * Clear the recovery variable to indicate to bootloader
 				 * that it is not required to start recovery again
 				 */
-				if (!software->globals.dry_run && software->bootloader_transaction_marker) {
-					unset_state((char*)BOOTVAR_TRANSACTION);
+				if (!software->parms.dry_run && software->bootloader_transaction_marker) {
+					bootloader_env_unset(BOOTVAR_TRANSACTION);
 				}
-				if (!software->globals.dry_run && save_state((char *)STATE_KEY, STATE_INSTALLED) != SERVER_OK) {
+				if (!software->parms.dry_run
+				    && software->bootloader_state_marker
+				    && save_state(STATE_INSTALLED) != SERVER_OK) {
 					ERROR("Cannot persistently store INSTALLED update state.");
 					notify(FAILURE, RECOVERY_ERROR, ERRORLEVEL, "Installation failed !");
 					inst.last_install = FAILURE;
@@ -621,6 +638,11 @@ void *network_initializer(void *data)
 
 		swupdate_progress_end(inst.last_install);
 
+		/*
+		 * Reload default values for update
+		 */
+		software->parms = parms;
+
 		pthread_mutex_lock(&stream_mutex);
 		inst.status = IDLE;
 		pthread_mutex_unlock(&stream_mutex);
@@ -645,7 +667,7 @@ void get_install_swset(char *buf, size_t len)
 	if (!buf)
 		return;
 
-	strncpy(buf, inst.software->software_set, len - 1);
+	strncpy(buf, inst.software->parms.software_set, len - 1);
 
 }
 
@@ -655,7 +677,7 @@ void get_install_running_mode(char *buf, size_t len)
 	if (!buf)
 		return;
 
-	strncpy(buf, inst.software->running_mode, len - 1);
+	strncpy(buf, inst.software->parms.running_mode, len - 1);
 }
 
 /*
@@ -671,3 +693,23 @@ int get_install_info(sourcetype *source, char *buf, size_t len)
 
 	return len;
 }
+
+void set_version_range(const char *minversion,
+		const char *maxversion, const char *current)
+{
+	if (minversion && strnlen(minversion, SWUPDATE_GENERAL_STRING_SIZE)) {
+		strlcpy(inst.software->minimum_version, minversion,
+			sizeof(inst.software->minimum_version));
+		inst.software->no_downgrading = true;
+	}
+	if (maxversion && strnlen(maxversion, SWUPDATE_GENERAL_STRING_SIZE)) {
+		strlcpy(inst.software->maximum_version, maxversion,
+			sizeof(inst.software->maximum_version));
+		inst.software->check_max_version = true;
+	}
+	if (current && strnlen(current, SWUPDATE_GENERAL_STRING_SIZE)) {
+		strlcpy(inst.software->current_version, current,
+			sizeof(inst.software->current_version));
+		inst.software->no_reinstalling = true;
+	}
+}
