@@ -35,7 +35,9 @@ typedef enum {
 	SWUPDATE_SUBPROCESS,
 	SET_AES_KEY,
 	SET_UPDATE_STATE,	/* set bootloader ustate */
-	REQ_INSTALL_EXT
+	GET_UPDATE_STATE,
+	REQ_INSTALL_EXT,
+	SET_VERSIONS_RANGE
 } msgtype;
 
 /*
@@ -67,6 +69,7 @@ struct swupdate_request {
 	char info[512];
 	char software_set[256];
 	char running_mode[256];
+	bool disable_store_swu;
 };
 
 typedef union {
@@ -99,6 +102,11 @@ typedef union {
 		char key_ascii[65]; /* Key size in ASCII (256 bit, 32 bytes bin) + termination */
 		char ivt_ascii[33]; /* Key size in ASCII (16 bytes bin) + termination */
 	} aeskeymsg;
+	struct {
+		char minimum_version[256];
+		char maximum_version[256];
+		char current_version[256];
+	} versions;
 } msgdata;
 	
 typedef struct {
@@ -127,7 +135,9 @@ int swupdate_async_start(writedata wr_func, getstatus status_func,
 				terminated end_func,
 				void *priv, ssize_t size);
 int swupdate_set_aes(char *key, char *ivt);
-
+int swupdate_set_version_range(const char *minversion,
+				const char *maxversion,
+				const char *currentversion);
 #ifdef __cplusplus
 }   // extern "C"
 #endif
