@@ -2,7 +2,7 @@
  * Author: Christian Storm
  * Copyright (C) 2016, Siemens AG
  *
- * SPDX-License-Identifier:     GPL-2.0-or-later
+ * SPDX-License-Identifier:     GPL-2.0-only
  */
 
 #include <stdio.h>
@@ -50,6 +50,8 @@ static struct option long_options[] = {
     {"interface", required_argument, NULL, 'f'},
     {"disable-token-for-dwl", no_argument, NULL, '1'},
     {"cache", required_argument, NULL, '2'},
+    {"initial-report-resend-period", required_argument, NULL, 'm'},
+	{"connection-timeout", required_argument, NULL, 's'},
     {NULL, 0, NULL, 0}};
 
 static unsigned short mandatory_argument_count = 0;
@@ -129,7 +131,8 @@ static channel_data_t channel_data_defaults = {.debug = false,
 					       .format = CHANNEL_PARSE_JSON,
 					       .nocheckanswer = false,
 					       .nofollow = false,
-					       .strictssl = true
+					       .strictssl = true,
+						   .connection_timeout = 0
 						};
 
 static struct timeval server_time;
@@ -740,7 +743,7 @@ server_op_res_t server_has_pending_action(int *action_id)
 		/*
 		 * Save the state
 		 */
-		if ((result = save_state((char *)STATE_KEY, STATE_OK)) != SERVER_OK) {
+		if ((result = save_state(STATE_OK)) != SERVER_OK) {
 			ERROR("Error while resetting update state on persistent "
 			"storage.\n");
 		}
@@ -891,7 +894,7 @@ server_op_res_t server_handle_initial_state(update_state_t stateovrrd)
 	/* NOTE (Re-)setting STATE_KEY=STATE_OK == '0' instead of deleting it
 	 *      as it may be required for the switchback/recovery U-Boot logics.
 	 */
-	if ((result = save_state((char *)STATE_KEY, STATE_OK)) != SERVER_OK) {
+	if ((result = save_state(STATE_OK)) != SERVER_OK) {
 		ERROR("Error while resetting update state on persistent "
 		      "storage.\n");
 		return result;
@@ -1596,9 +1599,13 @@ void server_print_help(void)
 	    "\t  -g, --gatewaytoken  Set gateway token.\n"
 	    "\t  -f, --interface     Set the network interface to connect to hawkBit.\n"
 	    "\t  --disable-token-for-dwl Do not send authentication header when downlloading SWU.\n"
-	    "\t  --cache <file>      Use cache file as starting SWU\n",
+	    "\t  --cache <file>      Use cache file as starting SWU\n"
+		"\t  -m, --initial-report-resend-period <seconds> Time to wait prior to retry "
+		"sending initial state with '-c' option (default: %ds).\n"
+		"\t  -s, --connection-timeout Set the server connection timeout (default: 300s).\n",
 	    CHANNEL_DEFAULT_POLLING_INTERVAL, CHANNEL_DEFAULT_RESUME_TRIES,
-	    CHANNEL_DEFAULT_RESUME_DELAY);
+	    CHANNEL_DEFAULT_RESUME_DELAY,
+	    INITIAL_STATUS_REPORT_WAIT_DELAY);
 }
 
 static int server_hawkbit_settings(void *elem, void  __attribute__ ((__unused__)) *data)
@@ -1624,11 +1631,17 @@ static int server_hawkbit_settings(void *elem, void  __attribute__ ((__unused__)
 	get_field(LIBCFG_PARSER, elem, "polldelay",
 		&server_hawkbit.polling_interval);
 
+	get_field(LIBCFG_PARSER, elem, "initial-report-resend-period",
+		&server_hawkbit.initial_report_resend_period);
+
 	suricatta_channel_settings(elem, &channel_data_defaults);
 
 	get_field(LIBCFG_PARSER, elem, "usetokentodwl",
 		&server_hawkbit.usetokentodwl);
 
+	get_field(LIBCFG_PARSER, elem, "connection-timeout",
+		&channel_data_defaults.connection_timeout);
+
 	GET_FIELD_STRING_RESET(LIBCFG_PARSER, elem, "targettoken", tmp);
 	if (strlen(tmp))
 		SETSTRING(server_hawkbit.targettoken, tmp);
@@ -1649,20 +1662,23 @@ server_op_res_t server_start(char *fname, int argc, char *argv[])
 
 	LIST_INIT(&server_hawkbit.configdata);
 
+	server_hawkbit.initial_report_resend_period = INITIAL_STATUS_REPORT_WAIT_DELAY;
 	if (fname) {
-		/*
-		 * Search "suricatta" section to be compatible with past
-		 */
-		read_module_settings(fname, "suricatta", server_hawkbit_settings,
-					NULL);
-		/*
-		 * Then try "hawkBit" because each server has its own
-		 * section
-		 */
-		read_module_settings(fname, "hawkbit", server_hawkbit_settings,
-					NULL);
-		read_module_settings(fname, "identify", settings_into_dict,
-					&server_hawkbit.configdata);
+		swupdate_cfg_handle handle;
+		swupdate_cfg_init(&handle);
+		if (swupdate_cfg_read_file(&handle, fname) == 0) {
+			/*
+			 * Search "suricatta" section to be compatible with past
+			 */
+			read_module_settings(&handle, "suricatta", server_hawkbit_settings, NULL);
+			/*
+			 * Then try "hawkBit" because each server has its own
+			 * section
+			 */
+			read_module_settings(&handle, "hawkbit", server_hawkbit_settings, NULL);
+			read_module_settings(&handle, "identify", settings_into_dict, &server_hawkbit.configdata);
+		}
+		swupdate_cfg_destroy(&handle);
 	}
 
 	if (loglevel >= DEBUGLEVEL) {
@@ -1673,7 +1689,7 @@ server_op_res_t server_start(char *fname, int argc, char *argv[])
 	/* reset to optind=1 to parse suricatta's argument vector */
 	optind = 1;
 	opterr = 0;
-	while ((choice = getopt_long(argc, argv, "t:i:c:u:p:xr:y::w:k:g:f:2:",
+	while ((choice = getopt_long(argc, argv, "t:i:c:u:p:xr:y::w:k:g:f:2:m:s:",
 				     long_options, NULL)) != -1) {
 		switch (choice) {
 		case 't':
@@ -1758,6 +1774,14 @@ server_op_res_t server_start(char *fname, int argc, char *argv[])
 		case '2':
 			SETSTRING(server_hawkbit.cached_file, optarg);
 			break;
+		case 'm':
+			server_hawkbit.initial_report_resend_period =
+				(unsigned int)strtoul(optarg, NULL, 10);
+			break;
+		case 's':
+			channel_data_defaults.connection_timeout =
+				(unsigned int)strtoul(optarg, NULL, 10);
+			break;
 		/* Ignore not recognized options, they can be already parsed by the caller */
 		case '?':
 			break;
@@ -1804,12 +1828,10 @@ server_op_res_t server_start(char *fname, int argc, char *argv[])
 	}
 	/* If an update was performed, report its status to the hawkBit server
 	 * prior to entering the main loop. May run indefinitely if server is
-	 * unavailable. In case of an error, the error is returned to the main
-	 * loop, thereby exiting suricatta. */
-	server_op_res_t state_handled;
+	 * unavailable.
+	 */
 	server_hawkbit.update_state = update_state;
 
-
 	/*
 	 * After a successful startup, a configData is always sent
 	 * Prepare default values
@@ -1821,15 +1843,10 @@ server_op_res_t server_start(char *fname, int argc, char *argv[])
 	 * by an external process and we have to wait for it
 	 */
 	if (update_state != STATE_WAIT) {
-		while ((state_handled = server_handle_initial_state(update_state)) !=
-		       SERVER_OK) {
-			if (state_handled == SERVER_EAGAIN) {
-				INFO("Sleeping for %ds until retrying...",
-				     INITIAL_STATUS_REPORT_WAIT_DELAY);
-				sleep(INITIAL_STATUS_REPORT_WAIT_DELAY);
-				continue;
-			}
-			return state_handled; /* Report error to main loop, exiting. */
+		while (server_handle_initial_state(update_state) != SERVER_OK) {
+			INFO("Sleeping for %ds until retrying...",
+				server_hawkbit.initial_report_resend_period);
+			sleep(server_hawkbit.initial_report_resend_period);
 		}
 	}
 
@@ -1945,7 +1962,7 @@ static server_op_res_t server_activation_ipc(ipc_message *msg)
 		/*
 		 * Save the state
 		 */
-		if ((result = save_state((char *)STATE_KEY, STATE_OK)) != SERVER_OK) {
+		if ((result = save_state(STATE_OK)) != SERVER_OK) {
 			ERROR("Error while resetting update state on persistent "
 			"storage.\n");
 		}
@@ -1959,6 +1976,19 @@ cleanup:
 	return result;
 }
 
+static void server_set_additional_device_attributes_ipc(json_object* json_data)
+{
+	for (int i = 0; i < json_object_array_length(json_data); ++i) {
+		json_object* attr_obj = json_object_array_get_idx(json_data, i);
+		json_object* attr_key_data = json_get_path_key(attr_obj, (const char *[]){"name", NULL});
+		json_object* attr_value_data = json_get_path_key(attr_obj, (const char *[]){"value", NULL});
+
+		const char* key = json_object_get_string(attr_key_data);
+		const char* value = json_object_get_string(attr_value_data);
+		dict_set_value(&server_hawkbit.configdata, key, value);
+	}
+}
+
 static server_op_res_t server_configuration_ipc(ipc_message *msg)
 {
 	struct json_object *json_root;
@@ -1981,6 +2011,14 @@ static server_op_res_t server_configuration_ipc(ipc_message *msg)
 			server_hawkbit.polling_interval_from_server = true;
 	}
 
+	json_data = json_get_path_key(
+		json_root, (const char*[]){"identify", NULL});
+
+	if (json_data) {
+		server_set_additional_device_attributes_ipc(json_data);
+		server_hawkbit.has_to_send_configData = true;
+	}
+
 	return SERVER_OK;
 }
 
